@@ -2,7 +2,7 @@
 /*
  * bonobo-mdi.c - implementation of a BonoboMDI object
  *
- * Copyright (C) 2001 Free Software Foundation
+ * Copyright (C) 2001-2002 Free Software Foundation
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +22,12 @@
  * Author: Paolo Maggi 
  */
 
+/*
+ * Modified by the gedit Team, 2001-2002. See the AUTHORS file for a 
+ * list of people on the gedit Team.  
+ * See the ChangeLog files for a list of changes. 
+ */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -43,6 +49,7 @@
 #define BONOBO_MDI_CHILD_KEY 	"BonoboMDIChild"
 #define UI_COMPONENT_KEY 	"UIComponent"
 #define BOOK_KEY		"Book"
+#define WINDOW_INFO_KEY		"BonoboMDIWindowInfo"
 
 static void            bonobo_mdi_class_init     (BonoboMDIClass  *);
 static void            bonobo_mdi_instance_init  (BonoboMDI *);
@@ -100,6 +107,7 @@ enum {
 	CHILD_CHANGED,
 	VIEW_CHANGED,
 	TOP_WINDOW_CREATED,
+	TOP_WINDOW_DESTROY,
 	ALL_WINDOWS_DESTROYED,
 	LAST_SIGNAL
 };
@@ -132,6 +140,9 @@ struct _BonoboMDIPrivate
 	
     	/* Paths for insertion of mdi-child list menu via */
 	gchar 		*child_list_path;
+
+	gint 		 default_window_height;
+	gint		 default_window_width;
 };
 
 typedef gboolean   (*BonoboMDISignal1) (GObject *, gpointer, gpointer);
@@ -255,6 +266,17 @@ bonobo_mdi_class_init (BonoboMDIClass *class)
 			      1, 
 			      BONOBO_TYPE_WINDOW);
 
+	mdi_signals[TOP_WINDOW_DESTROY] = 
+		g_signal_new ("top_window_destroy",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (BonoboMDIClass, top_window_destroy),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__OBJECT,
+			      G_TYPE_NONE, 
+			      1, 
+			      BONOBO_TYPE_WINDOW);
+
 	mdi_signals[ALL_WINDOWS_DESTROYED] = 
 		g_signal_new ("all_windows_destroyed",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -362,7 +384,8 @@ bonobo_mdi_instance_init (BonoboMDI *mdi)
  * A pointer to a new BonoboMDI object.
  **/
 GObject*
-bonobo_mdi_new (const gchar *mdi_name, const gchar *title) 
+bonobo_mdi_new (const gchar *mdi_name, const gchar *title,
+		gint default_window_width, gint default_window_height) 
 {
 	BonoboMDI *mdi;
 
@@ -373,6 +396,9 @@ bonobo_mdi_new (const gchar *mdi_name, const gchar *title)
 	mdi->priv->mdi_name = g_strdup (mdi_name);
 	mdi->priv->title    = g_strdup (title);
 
+	mdi->priv->default_window_width = default_window_width;
+	mdi->priv->default_window_height = default_window_height;
+
 	gedit_debug (DEBUG_MDI, "END");
 
 	return G_OBJECT (mdi);
@@ -484,11 +510,16 @@ child_list_menu_create (BonoboMDI *mdi, BonoboWindow *win)
 		gchar *verb_name = NULL;
 		gchar *tip;
 		gchar *escaped_name;
+		gchar *safe_name;
 		gchar *child_name = bonobo_mdi_child_get_name (BONOBO_MDI_CHILD (child->data));
 
-		escaped_name = escape_underscores (child_name);
-		
-		tip =  g_strdup_printf (_("Activate %s"), child_name);
+		safe_name = g_markup_escape_text (child_name, strlen (child_name));
+		g_return_if_fail (safe_name != NULL);
+
+		escaped_name = escape_underscores (safe_name);
+		g_return_if_fail (escaped_name != NULL);
+
+		tip =  g_strdup_printf (_("Activate %s"), safe_name);
 		verb_name = g_strdup_printf ("Child_%p", child->data);
 		xml = g_strdup_printf ("<menuitem name=\"%s\" verb=\"%s\""
 				" _label=\"%s\"/>", verb_name, verb_name, escaped_name);
@@ -497,6 +528,7 @@ child_list_menu_create (BonoboMDI *mdi, BonoboWindow *win)
 
 		g_free (tip);
 		g_free (child_name);
+		g_free (safe_name);
 		g_free (escaped_name);
 
 		bonobo_ui_component_set_translate (ui_component, mdi->priv->child_list_path, xml, NULL);
@@ -556,23 +588,39 @@ void
 child_list_menu_add_item (BonoboMDI *mdi, BonoboMDIChild *child)
 {
 	GList *win_node;
-	gchar *child_name, *escaped_name;
+	gchar *child_name, *escaped_name, *safe_name;
 	gchar* xml, *cmd, *verb_name, *tip;
+	int accel_num;
 	
 	if(mdi->priv->child_list_path == NULL)
 		return;
+
+	accel_num = g_list_length (mdi->priv->children);
 	
 	win_node = mdi->priv->windows;
 
 	child_name = bonobo_mdi_child_get_name (child);
-	escaped_name = escape_underscores (child_name);
+
+	safe_name = g_markup_escape_text (child_name, strlen (child_name));
+	g_return_if_fail (safe_name != NULL);
+
+	escaped_name = escape_underscores (safe_name);
+	g_return_if_fail (escaped_name != NULL);
+
 	verb_name = g_strdup_printf ("Child_%p", child);
 	
-	tip = g_strdup_printf (_("Activate %s"), child_name);
+	tip = g_strdup_printf (_("Activate %s"), safe_name);
 	xml = g_strdup_printf ("<menuitem name=\"%s\" verb=\"%s\""
 				" _label=\"%s\"/>", verb_name, verb_name, escaped_name);
-	cmd =  g_strdup_printf ("<cmd name = \"%s\" _label=\"%s\""
+
+	if (accel_num > 9)
+		cmd =  g_strdup_printf ("<cmd name = \"%s\" _label=\"%s\""
 				" _tip=\"%s\"/>", verb_name, escaped_name, tip);
+	else
+		cmd =  g_strdup_printf ("<cmd name = \"%s\" _label=\"%s\""
+			" _tip=\"%s\" accel=\"*Alt*%d\"/>", verb_name,
+			escaped_name, tip, accel_num);
+
 	
 	while (win_node) 
 	{
@@ -595,6 +643,7 @@ child_list_menu_add_item (BonoboMDI *mdi, BonoboMDIChild *child)
 	g_free (xml);
 	g_free (cmd);
 	g_free (verb_name);
+	g_free (safe_name);
 }
 
 static gboolean 
@@ -712,7 +761,7 @@ book_button_release (GtkWidget *widget, GdkEventButton *e, gpointer data)
 						win = BONOBO_WINDOW (gtk_widget_get_toplevel (
 									GTK_WIDGET (old_book)));
 						mdi->priv->windows = g_list_remove(mdi->priv->windows, win);
-						gtk_widget_destroy (GTK_WIDGET(win));
+						gtk_widget_destroy (GTK_WIDGET (win));
 					}					
 										
 					g_signal_emit (G_OBJECT (mdi), 
@@ -893,53 +942,70 @@ toplevel_focus (BonoboWindow *win, GdkEventFocus *event, BonoboMDI *mdi)
 	return FALSE;
 }
 
+static gboolean 
+app_configure_event_handler (GtkWidget *widget, GdkEventConfigure *event)
+{
+	BonoboMDIWindowInfo *window_info;
+
+	g_return_val_if_fail (BONOBO_IS_WINDOW (widget), FALSE);
+	g_return_val_if_fail (event != NULL, FALSE);
+	
+	window_info = (BonoboMDIWindowInfo*) 
+			g_object_get_data (G_OBJECT (widget), WINDOW_INFO_KEY);
+	g_return_val_if_fail (window_info != NULL, FALSE);
+
+	window_info->width = event->width;
+	window_info->height = event->height;
+
+	return FALSE;
+}
+
+static gboolean 
+app_window_state_event_handler (GtkWidget *widget, GdkEventWindowState *event)
+{
+	BonoboMDIWindowInfo *window_info;
+
+	g_return_val_if_fail (BONOBO_IS_WINDOW (widget), FALSE);
+	g_return_val_if_fail (event != NULL, FALSE);
+	
+	window_info = (BonoboMDIWindowInfo*) 
+			g_object_get_data (G_OBJECT (widget), WINDOW_INFO_KEY);
+	g_return_val_if_fail (window_info != NULL, FALSE);
+	
+	window_info->state = event->new_window_state;
+	
+	return FALSE;
+}
+
+
 static void 
 app_clone (BonoboMDI *mdi, BonoboWindow *win, const char *window_role)
 {
-	/*
-	BonoboDockLayout *layout;
-
-	
-	gchar *layout_string = NULL;
-	*/
-	
 	gedit_debug (DEBUG_MDI, "");
 	
-	/* FIXME */
-	/*
-	if (win != NULL) 
+	app_create (mdi, NULL, window_role);
+
+	if (win != NULL)
 	{
-		BonoboDock* dock = NULL;
-		GList *children_list = gtk_container_get_children (GTK_CONTAINER (win));
-	
-		while (children_list) 
+		const BonoboMDIWindowInfo *window_info = bonobo_mdi_get_window_info (win);
+		g_return_if_fail (window_info != NULL);
+
+		if ((window_info->state & GDK_WINDOW_STATE_MAXIMIZED) != 0)
+			gtk_window_maximize (GTK_WINDOW (mdi->priv->active_window));
+		else
 		{
-		
-			if (BONOBO_IS_DOCK (children_list->data))
-			{
-				dock = BONOBO_DOCK (children_list->data);
-				break;
-			}
+			gtk_window_set_default_size (GTK_WINDOW (mdi->priv->active_window), 
+					     window_info->width,
+					     window_info->height);
 			
-			children_list = children_list->next;
-		}		
-
-		if (dock != NULL)
-		{
-			layout = bonobo_dock_get_layout (dock);
-			if (layout)
-			layout_string = bonobo_dock_layout_create_string (layout);
-			g_object_unref (G_OBJECT (layout));
+			gtk_window_unmaximize (GTK_WINDOW (mdi->priv->active_window));
 		}
-	}
-	*/
-	
-	app_create (mdi, /*layout_string*/ NULL, window_role);
 
-	/*
-	if (layout_string)
-		g_free(layout_string);
-	*/
+		if ((window_info->state & GDK_WINDOW_STATE_STICKY ) != 0)
+			gtk_window_stick (GTK_WINDOW (mdi->priv->active_window));
+		else
+			gtk_window_unstick (GTK_WINDOW (mdi->priv->active_window));
+	}
 
 	gedit_debug (DEBUG_MDI, "END");
 }
@@ -1008,7 +1074,7 @@ app_close_book (BonoboWindow *win, GdkEventAny *event, BonoboMDI *mdi)
 			if (node == NULL) 
 			{   
 				/* all the views reside in this BonoboWindow */
-				g_signal_emit(G_OBJECT(mdi), 
+				g_signal_emit (G_OBJECT(mdi), 
 					      mdi_signals [REMOVE_CHILD],
 					      0,
 					      child, 
@@ -1063,11 +1129,13 @@ static void
 app_destroy (BonoboWindow *win, BonoboMDI *mdi)
 {
 	gedit_debug (DEBUG_MDI, "");
-
+	
 	if (mdi->priv->active_window == win)
 		mdi->priv->active_window = 
 			(mdi->priv->windows != NULL) ? BONOBO_WINDOW (mdi->priv->windows->data) : NULL;
 
+	g_signal_emit (G_OBJECT (mdi), mdi_signals [TOP_WINDOW_DESTROY], 0, win);
+
 	gedit_debug (DEBUG_MDI, "END");
 }
 
@@ -1112,14 +1180,19 @@ app_create (BonoboMDI *mdi, gchar *layout_string, const char *window_role)
 	GtkWidget *window;
 	BonoboWindow *bw;
 	gchar* config_path;
-	BonoboUIContainer* ui_container = NULL;
-  	BonoboUIComponent* ui_component = NULL;
+	BonoboUIContainer *ui_container = NULL;
+  	BonoboUIComponent *ui_component = NULL;
+	BonoboMDIWindowInfo *window_info = NULL;
 
 	gedit_debug (DEBUG_MDI, "");
 
 	window = bonobo_window_new (mdi->priv->mdi_name, mdi->priv->title);
 	g_return_if_fail (window != NULL);
 
+	gtk_window_set_default_size (GTK_WINDOW (window),
+				     mdi->priv->default_window_width,
+				     mdi->priv->default_window_height);
+
 	if (window_role)
 		gtk_window_set_role (GTK_WINDOW (window), window_role);
 	else {
@@ -1140,6 +1213,10 @@ app_create (BonoboMDI *mdi, gchar *layout_string, const char *window_role)
 			  G_CALLBACK (toplevel_focus), mdi);
 	g_signal_connect (G_OBJECT (window), "destroy",
 			  G_CALLBACK (app_destroy), mdi);
+	g_signal_connect (G_OBJECT (window), "configure_event",
+	                  G_CALLBACK (app_configure_event_handler), NULL);
+	g_signal_connect (G_OBJECT (window), "window_state_event",
+	                  G_CALLBACK (app_window_state_event_handler), NULL);
 
 	/* Create Container: */
  	ui_container = bonobo_window_get_ui_container (bw);
@@ -1178,7 +1255,10 @@ app_create (BonoboMDI *mdi, gchar *layout_string, const char *window_role)
 	mdi->priv->active_child = NULL;
 	mdi->priv->active_view = NULL;
 
+	window_info = g_new0 (BonoboMDIWindowInfo, 1);
+			
 	g_object_set_data (G_OBJECT (bw), UI_COMPONENT_KEY, ui_component);
+	g_object_set_data_full (G_OBJECT (bw), WINDOW_INFO_KEY, window_info, g_free);
 
 	g_signal_emit (G_OBJECT (mdi), mdi_signals [TOP_WINDOW_CREATED], 0, window);
 
@@ -1461,14 +1541,10 @@ bonobo_mdi_remove_view (BonoboMDI *mdi, GtkWidget *view, gint force)
 	if (g_list_length (GTK_NOTEBOOK (book)->children) == 1)
 		app_set_view (mdi, window, NULL);
 		
-	gedit_debug (DEBUG_VIEW, "PRIMA1: %d", G_OBJECT (view)->ref_count);
 	bonobo_mdi_child_remove_view (child, view);
-	gedit_debug (DEBUG_VIEW, "DOPO1: %d", G_OBJECT (view)->ref_count);
 
-	gedit_debug (DEBUG_VIEW, "PRIMA2: %d", G_OBJECT (view)->ref_count);
 	pn = gtk_notebook_page_num (GTK_NOTEBOOK (book), view);
 	gtk_notebook_remove_page (GTK_NOTEBOOK (book), pn);
-	gedit_debug (DEBUG_VIEW, "DOPO2");
 
 	if (GTK_NOTEBOOK (book)->cur_page == NULL) 
 	{
@@ -2071,7 +2147,8 @@ bonobo_mdi_get_view_from_window (BonoboMDI *mdi, BonoboWindow *win)
 }
 
 void 
-bonobo_mdi_construct (BonoboMDI *mdi, const gchar* name, const gchar* title)
+bonobo_mdi_construct (BonoboMDI *mdi, const gchar* name, const gchar* title,
+		gint default_window_width, gint default_window_height)
 {
 	g_return_if_fail (mdi->priv->mdi_name == NULL);
 	g_return_if_fail (mdi->priv->title == NULL);
@@ -2081,6 +2158,10 @@ bonobo_mdi_construct (BonoboMDI *mdi, const gchar* name, const gchar* title)
 
 	mdi->priv->mdi_name = g_strdup (name);
 	mdi->priv->title = g_strdup (title);
+
+	mdi->priv->default_window_width = default_window_width;
+	mdi->priv->default_window_height = default_window_height;
+
 }
 
 GList *
@@ -2107,3 +2188,10 @@ bonobo_mdi_get_ui_component_from_window (BonoboWindow* win)
 	return BONOBO_UI_COMPONENT (
 			g_object_get_data (G_OBJECT (win), UI_COMPONENT_KEY));
 }
+
+const BonoboMDIWindowInfo *
+bonobo_mdi_get_window_info (BonoboWindow *win)
+{
+	return (const BonoboMDIWindowInfo *) 
+			g_object_get_data (G_OBJECT (win), WINDOW_INFO_KEY);
+}
