@@ -870,7 +870,7 @@ gedit_document_load_from_stdin (GeditDocument* doc, GError **error)
 	while (feof (stdin) == 0)
 	{
 		buffer_length = fread (tmp_buf, 1, GEDIT_STDIN_BUFSIZE, stdin);
-		tmp_buf [buffer_length + 1] = '\0';
+		tmp_buf [buffer_length] = '\0';
 		g_string_append (file_contents, tmp_buf);
 
 		if (ferror (stdin) != 0)
@@ -892,10 +892,39 @@ gedit_document_load_from_stdin (GeditDocument* doc, GError **error)
 	{
 		if (!g_utf8_validate (file_contents->str, file_contents->len, NULL))
 		{
-			g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, GEDIT_ERROR_INVALID_UTF8_DATA,
-				_("Invalid UTF-8 data"));
+			/* The file contains invalid UTF8 data */
+			/* Try to convert it to UTF-8 from currence locale */
+			GError *conv_error = NULL;
+			gchar* converted_file_contents = NULL;
+			gsize bytes_written;
+			
+			converted_file_contents = g_locale_to_utf8 (file_contents->str, file_contents->len,
+					NULL, &bytes_written, &conv_error); 
+						
+			if ((conv_error != NULL) || 
+			    !g_utf8_validate (converted_file_contents, bytes_written, NULL))		
+			{
+
+				/* Coversion failed */	
+				if (conv_error != NULL)
+					g_error_free (conv_error);
+
+				g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, 
+					     GEDIT_ERROR_INVALID_UTF8_DATA,
+				     	     _("Invalid UTF-8 data"));
+				
+				if (converted_file_contents != NULL)
+					g_free (converted_file_contents);
+				
+				g_string_free (file_contents, TRUE);
+				
+				return FALSE;
+			}
+
 			g_string_free (file_contents, TRUE);
-			return FALSE;
+
+			/* FIXME: this could be more efficient */
+			file_contents = g_string_new (converted_file_contents);
 		}
 
 		gedit_undo_manager_begin_not_undoable_action (doc->priv->undo_manager);
@@ -1086,6 +1115,9 @@ follow_symlinks (const gchar *filename, GError **error)
 	gchar *followed_filename;
 	gint link_count;
 
+	g_return_val_if_fail (filename != NULL, NULL);
+	
+	gedit_debug (DEBUG_DOCUMENT, "Filename: %s", filename); 
 	g_return_val_if_fail (strlen (filename) + 1 <= GEDIT_MAX_PATH_LEN, NULL);
 
 	followed_filename = g_strdup (filename);
@@ -1229,16 +1261,8 @@ gedit_document_save_as_real (GeditDocument* doc, const gchar *uri,
 	}
 
 	/* Get filename from uri */
-	temp_filename = eel_make_uri_canonical (uri);
-	
-	if ((temp_filename == NULL) || (strlen (temp_filename) <= 7))
-		filename = NULL;
-	else
-		filename = g_strdup (temp_filename + 7);
+	filename = gnome_vfs_get_local_path_from_uri (uri);
 
-	if (temp_filename != NULL)
-		g_free (temp_filename);
-	
 	if (!filename)
 	{
 		g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, 0,
@@ -1810,6 +1834,7 @@ gedit_document_find (GeditDocument* doc, const gchar* str,
 	GtkTextIter iter;
 	gboolean found = FALSE;
 	GtkTextSearchFlags search_flags;
+	gchar *converted_str;
 
 	gedit_debug (DEBUG_DOCUMENT, "");
 
@@ -1817,6 +1842,12 @@ gedit_document_find (GeditDocument* doc, const gchar* str,
 	g_return_val_if_fail (doc->priv != NULL, FALSE);
 	g_return_val_if_fail (str != NULL, FALSE);
 
+	converted_str = gedit_utils_convert_search_text (str);
+	g_return_val_if_fail (converted_str != NULL, FALSE);
+	
+	gedit_debug (DEBUG_DOCUMENT, "str: %s", str);
+	gedit_debug (DEBUG_DOCUMENT, "converted_str: %s", converted_str);
+
 	search_flags = GTK_TEXT_SEARCH_VISIBLE_ONLY | GTK_TEXT_SEARCH_TEXT_ONLY;
 	
 	if (!case_sensitive)
@@ -1843,11 +1874,11 @@ gedit_document_find (GeditDocument* doc, const gchar* str,
 	else		
 		gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &iter, 0);
 
-	if (*str != '\0')
+	if (*converted_str != '\0')
     	{
       		GtkTextIter match_start, match_end;
 
-          	found = gedit_text_iter_forward_search (&iter, str, search_flags,
+          	found = gedit_text_iter_forward_search (&iter, converted_str, search_flags,
                                         	&match_start, &match_end,
                                                	NULL);	
 		if (found)
@@ -1866,6 +1897,8 @@ gedit_document_find (GeditDocument* doc, const gchar* str,
 		doc->priv->last_search_was_case_sensitive = case_sensitive;
 	}
 
+	g_free (converted_str);
+
 	return found;
 }
 
@@ -1956,12 +1989,15 @@ gedit_document_replace_selected_text (GeditDocument *doc, const gchar *replace)
 {
 	GtkTextIter iter;
 	GtkTextIter sel_bound;
+	gchar *converted_replace;
 
 	gedit_debug (DEBUG_DOCUMENT, "");
 
 	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
 	g_return_if_fail (replace != NULL);	
 
+	converted_replace = gedit_utils_convert_search_text (replace);
+
 	gtk_text_buffer_get_iter_at_mark (GTK_TEXT_BUFFER (doc),			
                                     &iter,
                                     gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
@@ -1991,10 +2027,10 @@ gedit_document_replace_selected_text (GeditDocument *doc, const gchar *replace)
                                     &iter,
                                     gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
 					                      "insert"));
-	if (*replace != '\0')
+	if (*converted_replace != '\0')
 		gtk_text_buffer_insert (GTK_TEXT_BUFFER (doc),
 					&iter,
-					replace, strlen (replace));
+					converted_replace, strlen (converted_replace));
 
 	if (doc->priv->last_replace_text != NULL)
 		g_free (doc->priv->last_replace_text);
@@ -2002,6 +2038,7 @@ gedit_document_replace_selected_text (GeditDocument *doc, const gchar *replace)
 	doc->priv->last_replace_text = g_strdup (replace);
 
 	gedit_document_end_user_action (doc);
+	g_free (converted_replace);
 }
 
 gboolean
