@@ -36,24 +36,46 @@
 #include <libgnomeui/libgnomeui.h>
 #include <libgnomevfs/gnome-vfs.h>
 
-#include "gnome-vfs-helpers.h"
+#include <eel/eel-vfs-extensions.h>
+
 #include "gedit2.h"
 #include "gedit-file.h"
 #include "gedit-debug.h"
 #include "gedit-utils.h"
 #include "gedit-mdi.h"
 #include "gedit-recent.h" 
-#include "gedit-prefs.h"
 #include "gedit-file-selector-util.h"
 #include "gedit-plugins-engine.h"
 #include "gnome-recent-model.h"
+#include "gedit-prefs-manager.h"
 
-static gboolean gedit_file_open_real (const gchar* file_name, GeditMDIChild* child);
-static gboolean gedit_file_save_as_real (const gchar* file_name, GeditMDIChild *child);
-
+static gchar 	*get_dirname_from_uri 		(const char *uri);
+static gboolean  gedit_file_open_real 		(const gchar* file_name, 
+						 GeditMDIChild* child);
+static gboolean  gedit_file_save_as_real 	(const gchar* file_name, 
+						 GeditMDIChild *child);
 
 static gchar* gedit_default_path = NULL;
 
+static gchar *
+get_dirname_from_uri (const char *uri)
+{
+	GnomeVFSURI *vfs_uri;
+	char *name;
+
+	/* Make VFS version of URI. */
+	vfs_uri = gnome_vfs_uri_new (uri);
+	if (vfs_uri == NULL) {
+		return NULL;
+	}
+
+	/* Extract name part. */
+	name = gnome_vfs_uri_extract_dirname (vfs_uri);
+	gnome_vfs_uri_unref (vfs_uri);
+
+	return name;
+}
+
 
 void 
 gedit_file_new (void)
@@ -134,29 +156,28 @@ gedit_file_open (GeditMDIChild *active_child)
 		gint i;
 		for (i = 0; files[i]; ++i)
 		{
+			gedit_debug (DEBUG_FILE, "[%d]: %s", i, files[i]);
+
 			if (gedit_file_open_real (files[i], active_child))
 			{
-				gchar *uri;
 				gchar *uri_utf8;
 				
-				uri = gnome_vfs_x_format_uri_for_display (files[i]);
-				uri_utf8 = g_filename_to_utf8 (uri, -1, NULL, NULL, NULL);
-				g_free (uri);
+				if (gedit_utils_uri_has_file_scheme (files[i]))
+				{				
+					if (gedit_default_path != NULL)
+						g_free (gedit_default_path);
+
+					gedit_default_path = get_dirname_from_uri (files[i]);
+				}
 				
+				uri_utf8 = eel_format_uri_for_display (files[i]);	
 				if (uri_utf8 != NULL)
 				{
 					gedit_utils_flash_va (_("Loaded file '%s'"), uri_utf8);
 
 					g_free (uri_utf8);
 				}
-
-				if (gedit_utils_uri_has_file_scheme (files[i]))
-				{				
-					if (gedit_default_path != NULL)
-						g_free (gedit_default_path);
-
-					gedit_default_path = gnome_vfs_x_uri_get_dirname (files[i]);
-				}				
+							
 			}
 			
 			gedit_debug (DEBUG_FILE, "File: %s", files[i]);
@@ -171,24 +192,15 @@ gedit_file_open_real (const gchar* file_name, GeditMDIChild* active_child)
 {
 	GError *error = NULL;
 	gchar *uri;
-	gchar *uri_utf8;
+	
 	GnomeRecentModel *recent;
 
 	gedit_debug (DEBUG_FILE, "File name: %s", file_name);
 
-	uri = gnome_vfs_x_make_uri_canonical (file_name);
+	uri = eel_make_uri_canonical (file_name);
 	g_return_val_if_fail (uri != NULL, FALSE);
-
-	uri_utf8 = g_filename_to_utf8 (uri, -1, NULL, NULL, NULL);
-	if (uri_utf8 == NULL)
-	{
-		g_free (uri);
-
-		return FALSE;
-	}
+	gedit_debug (DEBUG_FILE, "Canonical uri: %s", uri);
 	
-	gedit_debug (DEBUG_FILE, "URI: %s", uri);
-
 	if (active_child == NULL ||
 	    !gedit_document_is_untouched (active_child->document))	     
 	{
@@ -199,7 +211,7 @@ gedit_file_open_real (const gchar* file_name, GeditMDIChild* active_child)
 
 		if (error)
 		{
-			gedit_utils_error_reporting_loading_file (uri_utf8, error,
+			gedit_utils_error_reporting_loading_file (uri, error,
 					GTK_WINDOW (gedit_get_active_window ()));
 			
 			g_error_free (error);
@@ -224,7 +236,7 @@ gedit_file_open_real (const gchar* file_name, GeditMDIChild* active_child)
 
 		if (error)
 		{
-			gedit_utils_error_reporting_loading_file (uri_utf8, error,
+			gedit_utils_error_reporting_loading_file (uri, error,
 					GTK_WINDOW (gedit_get_active_window ()));
 			
 			g_error_free (error);
@@ -234,9 +246,8 @@ gedit_file_open_real (const gchar* file_name, GeditMDIChild* active_child)
 	}
 	
 	recent = gedit_recent_get_model ();
-	gnome_recent_model_add (recent, uri_utf8);
+	gnome_recent_model_add (recent, uri);
 
-	g_free (uri_utf8);
 	g_free (uri);
 
 	gedit_debug (DEBUG_FILE, "END: %s\n", file_name);
@@ -268,9 +279,23 @@ gedit_file_save (GeditMDIChild* child)
 
 	if (!gedit_document_get_modified (doc))	
 	{
+		gchar *raw_uri;
+		gboolean deleted = FALSE;
+		
 		gedit_debug (DEBUG_FILE, "Not modified");
 
-		return TRUE;
+		raw_uri = gedit_document_get_raw_uri (doc);
+		if (raw_uri != NULL)
+		{
+			if (gedit_document_is_readonly (doc))
+				deleted = FALSE;
+			else
+				deleted = !gedit_utils_uri_exists (raw_uri);
+		}
+		g_free (raw_uri);
+			
+		if (!deleted)
+			return TRUE;
 	}
 	
 	uri = gedit_document_get_uri (doc);
@@ -307,9 +332,7 @@ gedit_file_save (GeditMDIChild* child)
 	{
 		GnomeRecentModel *recent;
 		gchar *raw_uri;
-		gchar *canonical_uri;
-		gchar *canonical_uri_utf8;
-
+		
 		gedit_debug (DEBUG_FILE, "OK");
 
 		gedit_utils_flash_va (_("File '%s' saved."), uri);
@@ -319,21 +342,11 @@ gedit_file_save (GeditMDIChild* child)
 		raw_uri = gedit_document_get_raw_uri (doc);
 		g_return_val_if_fail (raw_uri != NULL, TRUE);
 		
-		canonical_uri = gnome_vfs_x_make_uri_canonical (raw_uri);
-		g_return_val_if_fail (canonical_uri != NULL, TRUE);
-
-		/* canonical_uri is not valid utf8 */
-		
-		canonical_uri_utf8 = g_filename_to_utf8 (canonical_uri, -1, NULL, NULL, NULL);
-		g_return_val_if_fail (canonical_uri_utf8 != NULL, TRUE);
-
 		recent = gedit_recent_get_model ();
-		gnome_recent_model_add (recent, canonical_uri_utf8);
+		gnome_recent_model_add (recent, raw_uri);
 
 		g_free (raw_uri);
-		g_free (canonical_uri);
-		g_free (canonical_uri_utf8);
-
+		
 		return TRUE;
 	}
 }
@@ -346,7 +359,7 @@ gedit_file_save_as (GeditMDIChild *child)
 	GeditDocument *doc;
 	gchar *fname = NULL;
 	gchar *path = NULL;
-	gchar *uri = NULL;
+	gchar *raw_uri = NULL;
 	
 	gedit_debug (DEBUG_FILE, "");
 
@@ -355,21 +368,24 @@ gedit_file_save_as (GeditMDIChild *child)
 	doc = child->document;
 	g_return_val_if_fail (doc != NULL, FALSE);
 
-	uri = gedit_document_get_uri (doc);
-	g_return_val_if_fail (uri != NULL, FALSE);
+	raw_uri = gedit_document_get_raw_uri (doc);
 
 	if (gedit_document_is_untitled (doc))
 	{
 		path = (gedit_default_path != NULL) ? 
 			g_strdup (gedit_default_path) : NULL;
-		fname = uri;
+
+		/* FIXME: check the case in which "Untitled n" is a non-ascii string */
+		fname = gedit_document_get_uri (doc);		
 	}
 	else
 	{
-		fname = gnome_vfs_x_uri_get_basename (uri);
+		g_return_val_if_fail (raw_uri != NULL, FALSE);
 
-		if (gedit_utils_uri_has_file_scheme (uri))
-			path = gnome_vfs_x_uri_get_dirname (uri);
+		fname = eel_uri_get_basename (raw_uri);
+
+		if (gedit_utils_uri_has_file_scheme (raw_uri))
+			path = get_dirname_from_uri (raw_uri);
 		else
 			path = (gedit_default_path != NULL) ? 
 				g_strdup (gedit_default_path) : NULL;
@@ -385,7 +401,7 @@ gedit_file_save_as (GeditMDIChild *child)
 			path,
 			fname);
 	
-	g_free (uri);
+	g_free (raw_uri);
 	g_free (fname);
 	
 	if (path != NULL)
@@ -393,33 +409,38 @@ gedit_file_save_as (GeditMDIChild *child)
 
 	if (file != NULL) 
 	{
+		gchar *uri;
 		gchar *file_utf8;
 
-		file_utf8 = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
+		uri = eel_make_uri_from_shell_arg (file);
+		g_return_val_if_fail (uri != NULL, FALSE);
 
+		file_utf8 = eel_format_uri_for_display (uri);
 		if (file_utf8 != NULL)
 			gedit_utils_flash_va (_("Saving file '%s' ..."), file_utf8);
 		
-		ret = gedit_file_save_as_real (file, child);
+		ret = gedit_file_save_as_real (uri, child);
 		
 		if (ret)
-		{
-			if (file_utf8 != NULL)
-				gedit_utils_flash_va (_("File '%s' saved."), file_utf8);
-
+		{			
 			if (gedit_default_path != NULL)
 				g_free (gedit_default_path);
 
-			gedit_default_path = gnome_vfs_x_uri_get_dirname (file);
+			gedit_default_path = get_dirname_from_uri (file);
+
+			if (file_utf8 != NULL)
+				gedit_utils_flash_va (_("File '%s' saved."), file_utf8);
 		}
 		else
 			gedit_utils_flash_va (_("The document has not been saved."));
 
 		gedit_debug (DEBUG_FILE, "File: %s", file);
+		g_free (uri);
 		g_free (file);
 
 		if (file_utf8 != NULL)
 			g_free (file_utf8);
+		
 	}
 
 	return ret;
@@ -440,7 +461,7 @@ gedit_file_save_as_real (const gchar* file_name, GeditMDIChild *child)
 	doc = child->document;
 	g_return_val_if_fail (doc != NULL, FALSE);
 
-	uri = gnome_vfs_x_make_uri_canonical (file_name);
+	uri = eel_make_uri_canonical (file_name);
 	g_return_val_if_fail (uri != NULL, FALSE);
 	
 	ret = gedit_document_save_as (doc, uri, &error);
@@ -449,7 +470,7 @@ gedit_file_save_as_real (const gchar* file_name, GeditMDIChild *child)
 	{
 		g_return_val_if_fail (error != NULL, FALSE);
 
-		gedit_utils_error_reporting_saving_file (uri, error,
+		gedit_utils_error_reporting_saving_file (file_name, error,
 					GTK_WINDOW (gedit_get_active_window ()));
 
 		g_error_free (error);
@@ -461,12 +482,12 @@ gedit_file_save_as_real (const gchar* file_name, GeditMDIChild *child)
 	}	
 	else
 	{
+#if 0
 		gchar *temp;
 
 		gedit_debug (DEBUG_FILE, "OK");
 
-		/* uri is not valid utf8 */
-		
+		/* uri is not valid utf8 */	
 		temp = g_filename_to_utf8 (uri, -1, NULL, NULL, NULL);
 
 		if (temp != NULL)
@@ -479,7 +500,12 @@ gedit_file_save_as_real (const gchar* file_name, GeditMDIChild *child)
 			g_free (temp);
 		}
 
-		
+#endif
+		GnomeRecentModel *recent;
+
+		recent = gedit_recent_get_model ();
+		gnome_recent_model_add (recent, uri);
+
 		g_free (uri);
 
 		return TRUE;
@@ -516,14 +542,20 @@ gedit_file_exit (void)
 	
 	gedit_plugins_engine_save_settings ();
 	
-	gedit_prefs_save_settings ();
-
+	gedit_prefs_manager_shutdown ();
+	
 	gedit_debug (DEBUG_FILE, "Unref gedit_mdi.");
 
 	g_object_unref (G_OBJECT (gedit_mdi));
 
 	gedit_debug (DEBUG_FILE, "Unref gedit_mdi: DONE");
 
+	gedit_debug (DEBUG_FILE, "Unref gedit_app_server.");
+
+	bonobo_object_unref (gedit_app_server);
+
+	gedit_debug (DEBUG_FILE, "Unref gedit_app_server: DONE");
+
 	gtk_main_quit ();
 }
 
@@ -583,7 +615,7 @@ gedit_file_revert (GeditMDIChild *child)
 					GTK_WINDOW (gedit_get_active_window ()));
 
 		g_error_free (error);
-		
+
 		gedit_utils_flash_va (_("The document has not been reverted."));
 
 		g_free (uri);
@@ -602,32 +634,11 @@ gedit_file_revert (GeditMDIChild *child)
 	}
 }
 
-static gchar *
-gedit_file_make_uri_from_shell_arg (const char *location)
-{
-	gchar* uri;
-	gchar* canonical_uri;
-	
-	gedit_debug (DEBUG_FILE, "LOCATION: %s", location);
-	
-	uri = gnome_vfs_x_make_uri_from_shell_arg (location);
-
-	gedit_debug (DEBUG_FILE, "URI: %s", uri);
-
-	canonical_uri = gnome_vfs_unescape_string_for_display (uri);
-
-	gedit_debug (DEBUG_FILE, "CANONICAL URI: %s", canonical_uri);
-
-	g_free (uri);
-
-	return canonical_uri;
-}
-
 gboolean 
-gedit_file_open_uri_list (GList* uri_list, gint line)
+gedit_file_open_uri_list (GList* uri_list, gint line, gboolean create)
 {
 	gchar *full_path;
-	gboolean ret = TRUE;
+	gboolean ret = FALSE;
 	gint loaded_files = 0;
 	BonoboMDIChild *active_child = NULL;
 	GeditView* view = NULL;
@@ -641,21 +652,76 @@ gedit_file_open_uri_list (GList* uri_list, gint line)
 	l = g_list_length (uri_list);
 	
 	if (l > 1)
-		gedit_utils_flash_va (_("Loading %d file..."), l);
+		gedit_utils_flash_va (_("Loading %d files..."), l);
 
 	active_child = bonobo_mdi_get_active_child (BONOBO_MDI (gedit_mdi));
 
         /* create a file for each document in the parameter list */
 	for ( ; uri_list; uri_list = g_list_next (uri_list))
 	{
-		full_path = gedit_file_make_uri_from_shell_arg (uri_list->data);
+		full_path = eel_make_uri_from_shell_arg (uri_list->data);
+	
 		if (full_path != NULL) 
 		{
-			if (gedit_file_open_real (full_path, 
-					(active_child != NULL) ? GEDIT_MDI_CHILD (active_child): NULL))
+			if (!gedit_utils_uri_has_file_scheme (full_path) || gedit_utils_uri_exists (full_path))
 			{
-				++loaded_files;
-				ret |= TRUE;
+				if (gedit_file_open_real (full_path, 
+					(active_child != NULL) ? GEDIT_MDI_CHILD (active_child): NULL))
+				{
+					++loaded_files;
+					ret |= TRUE;
+				}
+			}
+			else
+			{
+				if (create)
+				{
+					GtkWidget *dialog;
+					gchar *str;
+					gboolean created = FALSE;
+					gchar *formatted_path;
+
+					formatted_path = eel_format_uri_for_display (full_path);
+					g_return_val_if_fail (formatted_path != NULL, FALSE);
+					
+					str = g_strdup_printf (_("The file \"%s\" does not exist. Would you like to create it?"),
+							       formatted_path);
+		
+					g_free (formatted_path);
+
+					dialog = gtk_message_dialog_new (
+						GTK_WINDOW (bonobo_mdi_get_active_window (BONOBO_MDI (gedit_mdi))),
+						GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+					   	GTK_MESSAGE_QUESTION,
+					   	GTK_BUTTONS_YES_NO,
+						str);
+
+					gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_NO);
+
+					gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+
+					if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES)
+					{
+						created = gedit_utils_create_empty_file (full_path);
+
+						if (!created)	
+							gedit_utils_error_reporting_creating_file (full_path, errno, GTK_WINDOW (dialog));
+					}
+									
+					gtk_widget_destroy (dialog);
+
+					g_free (str);
+					
+					if (created)
+					{
+						if (gedit_file_open_real (full_path, 
+							(active_child != NULL) ? GEDIT_MDI_CHILD (active_child): NULL))
+						{
+							++loaded_files;
+							ret |= TRUE;
+						}
+					}
+				}
 			}
 			
 			g_free (full_path);
@@ -687,22 +753,19 @@ gedit_file_open_recent (GnomeRecentView *view, const gchar *uri, gpointer data)
 {
 	gboolean ret = FALSE;
 	GeditView* active_view;
-	gchar *temp;
 
 	gedit_debug (DEBUG_FILE, "Open : %s", uri);
 
-	temp = g_filename_from_utf8 (uri, -1, NULL, NULL, NULL);
+	/* Note that gedit_file_open_single_uri takes a possibly mangled "uri", in UTF8 */
 
-	if (temp != NULL)
-	{
-		ret = gedit_file_open_single_uri (temp);
+	ret = gedit_file_open_single_uri (uri);
 	
-		if (ret) {
-			GnomeRecentModel *model;
+	if (!ret) 
+	{
+		GnomeRecentModel *model;
 
-			model = gedit_recent_get_model ();
-			gnome_recent_model_add (model, uri);
-		}
+		model = gedit_recent_get_model ();
+		gnome_recent_model_delete (model, uri);
 	}
 		
 	active_view = gedit_get_active_view ();
@@ -714,24 +777,9 @@ gedit_file_open_recent (GnomeRecentView *view, const gchar *uri, gpointer data)
 	return ret;
 }
 
-
-static gchar *
-gedit_file_make_canonical_uri_from_input (const char *location)
-{
-	gchar* uri;
-	gchar* canonical_uri;
-	
-	gedit_debug (DEBUG_FILE, "");
-	
-	uri = gnome_vfs_x_make_uri_from_input (location);
-
-	canonical_uri = gnome_vfs_x_make_uri_canonical (uri);
-
-	g_free (uri);
-
-	return canonical_uri;
-}
-
+/*
+ *  uri: a possibly mangled "uri", in UTF8
+ */
 gboolean 
 gedit_file_open_single_uri (const gchar* uri)
 {
@@ -742,7 +790,7 @@ gedit_file_open_single_uri (const gchar* uri)
 	
 	if (uri == NULL) return FALSE;
 	
-	full_path = gedit_file_make_canonical_uri_from_input (uri);
+	full_path = eel_make_uri_from_input (uri);
 
 	if (full_path != NULL) 
 	{
@@ -756,8 +804,8 @@ gedit_file_open_single_uri (const gchar* uri)
 		{
 			gchar *uri_utf8;
 
-			uri_utf8 = g_filename_to_utf8 (full_path, -1, NULL, NULL, NULL);
-
+			uri_utf8 = eel_format_uri_for_display (full_path);
+			
 			if (uri_utf8 != NULL)
 			{
 				gedit_utils_flash_va (_("Loaded file '%s'"), uri_utf8);
